package application;

import javafx.beans.property.DoubleProperty;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.GridPane;
import javafx.scene.text.TextAlignment;

/**
 * @author David Brown
 * @version 2017-11-24
 */
public class ButtonPane extends GridPane {

    // ---------------------------------------------------------------
    /**
     * Adds an EventHandler to a button. Updates the appropriate base or height
     * property.
     */
    private class ButtonHandler implements EventHandler<ActionEvent> {
	// The amount to increment the base or height.
	private double direction = 0;
	private DoubleProperty property = null;

	public ButtonHandler(final double direction,
		final DoubleProperty property) {
	    this.direction = direction;
	    this.property = property;
	}

	@Override
	public void handle(final ActionEvent event) {
	    this.property.set(this.property.get() + this.direction);
	}
    }

    // ---------------------------------------------------------------
    /**
     * The format string for reading / displaying numeric input / output. Five
     * digits, one decimal.
     */
    private static final String formatString = "%5.1f";

    private Triangle model = null;

    private final Button baseUp = new Button("+");
    private final Button baseDown = new Button("-");
    private final Button heightUp = new Button("+");
    private final Button heightDown = new Button("-");
    private final Label labelBase = new Label();
    private final Label labelHeight = new Label();
    private final Label labelHypotenuse = new Label();

    /**
     * @param model
     */
    public ButtonPane(final Triangle model) {
	this.model = model;
	this.layoutPane();
	this.registerListeners();
    }

    /**
     *
     */
    private void layoutPane() {
	this.labelBase.setTextAlignment(TextAlignment.RIGHT);
	this.labelHeight.setTextAlignment(TextAlignment.RIGHT);
	this.labelHypotenuse.setTextAlignment(TextAlignment.RIGHT);

	this.setAlignment(Pos.CENTER);
	this.setHgap(10);
	this.setVgap(10);
	this.setPadding(new Insets(25, 25, 25, 25));

	// Add first row.
	this.add(new Label("Base:"), 0, 0);
	this.add(this.baseUp, 1, 0);
	this.add(this.baseDown, 2, 0);
	this.add(this.labelBase, 3, 0);
	// Add second row.
	this.add(new Label("Height: "), 0, 1);
	this.add(this.heightUp, 1, 1);
	this.add(this.heightDown, 2, 1);
	this.add(this.labelHeight, 3, 1);
	// Add third row.
	this.add(new Label("Hypotenuse:"), 0, 2);
	this.add(this.labelHypotenuse, 3, 2);
    }

    /**
     *
     */
    private void registerListeners() {

	// Add the button listeners.
	this.baseUp
		.setOnAction(new ButtonHandler(1, this.model.baseProperty()));
	this.baseDown
		.setOnAction(new ButtonHandler(-1, this.model.baseProperty()));
	this.heightUp
		.setOnAction(new ButtonHandler(1, this.model.heightProperty()));
	this.heightDown.setOnAction(
		new ButtonHandler(-1, this.model.heightProperty()));

	// Bind the buttons' disable properties.
	this.baseUp.disableProperty().bind(
		model.baseProperty().greaterThanOrEqualTo(Triangle.MAX_SIDE));
	this.baseDown.disableProperty()
		.bind(model.baseProperty().lessThanOrEqualTo(0));

	this.heightUp.disableProperty().bind(
		model.heightProperty().greaterThanOrEqualTo(Triangle.MAX_SIDE));
	this.heightDown.disableProperty()
		.bind(model.heightProperty().lessThanOrEqualTo(0));
	// Bind the labels to the model fields.
	this.labelBase.textProperty()
		.bind(model.baseProperty().asString(formatString));
	this.labelHeight.textProperty()
		.bind(model.heightProperty().asString(formatString));
	this.labelHypotenuse.textProperty()
		.bind(model.hypotenuseProperty().asString(formatString));
    }

}
